/********************************************************************************
NeoTiles v1.0 MAP EDITOR

my attempt at a map editor for my isometric tileset

********************************************************************************/
RequireScript("a_req.js");
RequireScript("b_cube.js");
RequireScript("neo_font.js");

// get width & height
// KEY_Q: quit w/o save
// KEY_S: save
// arrows: move camera
// [: zoom out
// ]: zoom in
// KEY_DELETE: x--
// KEY_END: x++
// KEY_INSERT: x--
// KEY_HOME: x++
// KEY_PGUP: type++
// KEY_PGDN: type--

function mapEditor() {
 this.keys = [];
 this.keys["typeUp"] = KEY_APOSTROPHE;
 this.keys["typeDn"] = KEY_SEMICOLON;
 this.keys["zMinus"] = KEY_PAGEDOWN;
 this.keys["zPlus"] = KEY_PAGEUP;
 this.keys["yMinus"] = KEY_HOME;
 this.keys["yPlus"] = KEY_END;
 this.keys["xMinus"] = KEY_INSERT;
 this.keys["xPlus"] = KEY_DELETE;
 this.keys["zoomOut"] = KEY_OPENBRACE;
 this.keys["zoomIn"] = KEY_CLOSEBRACE;
 this.keys["add"] = KEY_SPACE;
 this.keys["del"] = KEY_BACKSPACE;
 this.bg = null;
 this.map = null;
 this.o = null;
 this.f = new Font(LoadFont("msmincho.rfn"));
 this.done = false;
 this.debug = false;
 this.help = false;
 }
mapEditor.prototype.save = function(fn) {
 // show interface (do you want to save/overwrite? etc)
 this.map.save(fn);
 // show interface (map "fn" saved!)
 }
mapEditor.prototype.prerender = function() {
 // blit bg
 if (this.bg) this.bg.blit(0,0);
 var u = g.unit*g.zoom;
 var w = u*this.map.width;
 var h = u*this.map.height;
 Line(g.camX,g.camY,g.camX,g.camY-u*this.map.layers.length,g.cols.w);
 Line(g.camX+w,g.camY-w/2,
		g.camX+w,g.camY-w/2-u*this.map.layers.length,g.cols.w);
 Triangle(g.camX,g.camY,g.camX+w,g.camY-w/2,g.camX+h,g.camY+h/2,g.cols.grey);
 Triangle(g.camX+w,g.camY-w/2,g.camX+w+h,g.camY-w/2+h/2,g.camX+h,g.camY+h/2,g.cols.grey);
 for (var i=0; i<this.map.layers.length; i++) {
  Line(g.camX,g.camY-i*u,
		g.camX+w,g.camY-i*u-w/2,g.cols.silv);
  Line(g.camX+w,g.camY-i*u-w/2,
		g.camX+w+h,g.camY-i*u-w/2+h/2,g.cols.w);
  }
 }
mapEditor.prototype.postrender = function() {
 // blit help text
 if (this.help) {
  this.f.drawText(SW/2,10,"Press H to turn off help");
  var KEY = keyStr;
  var msg = "";
  msg += "X: -"+KEY(this.keys["xMinus"],0)+" +"+KEY(this.keys["xPlus"],0)+"\n";
  msg += "Y: -"+KEY(this.keys["yMinus"],0)+" +"+KEY(this.keys["yPlus"],0)+"\n";
  msg += "Z: -"+KEY(this.keys["zMinus"],0)+" +"+KEY(this.keys["zPlus"],0)+"\n";
  msg += "T: -"+KEY(this.keys["typeDn"],0)+" +"+KEY(this.keys["typeUp"],0)+"\n";
  msg += "A: "+KEY(this.keys["add"],0)+"\n";
  this.f.drawTextBox(10,10,150,100,0,msg);
  }
 else {
  this.f.drawText(SW/2,10,"Press H for help");
  }
 Rectangle(0,SH-30, 255,30, g.cols.k);
 Line(0,SH-30, 0,SH, g.cols.w);
 Line(255,SH-30, 255,SH, g.cols.w);
 var r1 = CreateColor(0,this.o.col.g,this.o.col.b), r2 = CreateColor(255,this.o.col.g,this.o.col.b);
 var g1 = CreateColor(this.o.col.r,0,this.o.col.b), g2 = CreateColor(this.o.col.r,255,this.o.col.b);
 var b1 = CreateColor(this.o.col.r,this.o.col.g,0), b2 = CreateColor(this.o.col.r,this.o.col.g,255);
 var c = CreateColor(this.o.col.r,this.o.col.g,this.o.col.b);
 for (var i=0; i<5; i++) {
  GradientLine(0,SH-30+1+i, 255,SH-30+1+i, r2,r1);
  GradientLine(0,SH-30+11+i, 255,SH-30+11+i, g2,g1);
  GradientLine(0,SH-30+21+i, 255,SH-30+21+i, b2,b1);
  }
 g.rgb.blit(this.o.col.r-2,SH-30+1-3);
 g.rgb.blit(this.o.col.g-2,SH-30+11-3);
 g.rgb.blit(this.o.col.b-2,SH-30+21-3);
 g.f.sq.drawText(256,SH-30, parseInt(this.o.col.r));
 g.f.sq.drawText(256,SH-30+10, parseInt(this.o.col.g));
 g.f.sq.drawText(256,SH-30+20, parseInt(this.o.col.b));
 Rectangle(SW-30,SH-30, 30,30, c);
 }
mapEditor.prototype.execute = function(fn) {
 this.o = new Object();
 var types = ["c","s0","s1","s2","s3","t0","t1","t2","t3"];
 var x = 0, y = 0, z = 0, type = 0;
 this.o.type = type;
 this.o.obj = null;
 this.o.col = new Object(); this.o.col.r = 255; this.o.col.g = 255; this.o.col.b = 255;
 this.map = new map2d(fn);
 if (this.map.layers.isEmpty()) {
  this.prerender();
  this.f.drawText(10,40,"Enter width:");
  this.map.width = this.f.input(70,40,2);
  this.f.drawText(10,50,"Enter height:");
  this.map.height = this.f.input(70,50,2);
  //this.f.drawText(10,60,"Enter layers:");
  //map.layers
  }
 while (!this.done) {
  var u = g.unit*g.zoom;
  var w = u*this.map.width;
  var h = u*this.map.height;
  if (this.o.type!=type||!this.o.obj) {
   this.o.type = types[type];
   switch (this.o.type.charAt(0)) {
    case "s": this.o.obj = new slope(1,x,y,z,null,this.o.type.charAt(1)); break;
    case "t": this.o.obj = new tri(1,x,y,z,null,this.o.type.charAt(1)); break;
    default: this.o.obj = new cube(1,x,y,z,null); break;
    }
   }
  if (this.o.obj.x!=x) this.o.obj.x = x;
  if (this.o.obj.y!=y) this.o.obj.y = y;
  if (this.o.obj.z!=z) this.o.obj.z = z;
  //**PRERENDER
  this.prerender();
  //**END PRERENDER
  this.map.blit(g.camX,g.camY);
  this.o.obj.blit(g.camX,g.camY,CreateColor(this.o.col.r,this.o.col.g,this.o.col.b));
  //**POSTRENDER
  for (var i=0; i<this.map.layers.length; i++) {
   Line(g.camX,g.camY-i*u,
		g.camX+h,g.camY-i*u+h/2,g.cols.w);
   Line(g.camX+h,g.camY-i*u+h/2,
		g.camX+w+h,g.camY-i*u-w/2+h/2,g.cols.silv);
   }
  Line(g.camX+h,g.camY+h/2,
		g.camX+h,g.camY+h/2-u*this.map.layers.length,g.cols.w);
  Line(g.camX+w+h,g.camY-w/2+h/2,
		g.camX+w+h,g.camY-w/2+h/2-u*this.map.layers.length,g.cols.w);
  if (this.debug) {
   var msg = "";
   msg += "["+this.o.obj.z+":"+this.o.obj.x+","+this.o.obj.y+"]\n";
   msg += "W:"+this.map.width+"\n";
   msg += "H:"+this.map.height+"\n";
   msg += "L:"+this.map.layers.length+"\n";
   this.f.drawTextBox(SW/2,20,150,100,0,msg);
   }
  this.postrender();
  //**END POSTRENDER
  FlipScreen();
  if (IsKeyPressed(KEY_E)) this.o.col.r -= 1;
  if (IsKeyPressed(KEY_R)) this.o.col.r += 1;
  if (this.o.col.r<0) this.o.col.r = 0; if (this.o.col.r>255) this.o.col.r = 255;
  if (IsKeyPressed(KEY_F)) this.o.col.g -= 1;
  if (IsKeyPressed(KEY_G)) this.o.col.g += 1;
  if (this.o.col.g<0) this.o.col.g = 0; if (this.o.col.g>255) this.o.col.g = 255;
  if (IsKeyPressed(KEY_V)) this.o.col.b -= 1;
  if (IsKeyPressed(KEY_B)) this.o.col.b += 1;
  if (this.o.col.b<0) this.o.col.b = 0; if (this.o.col.b>255) this.o.col.b = 255;
  if (IsKeyPressed(this.keys["typeDn"])) type -= 1;
  if (IsKeyPressed(this.keys["typeUp"])) type += 1;
  if (type<0) type = types.length+type; type = type%types.length;
  if (IsKeyPressed(this.keys["xMinus"])) x -= 1;
  if (IsKeyPressed(this.keys["xPlus"])) x += 1;
  if (x<0) x = 0; if (x>=this.map.width) x = this.map.width-1;
  if (IsKeyPressed(this.keys["yMinus"])) y -= 1;
  if (IsKeyPressed(this.keys["yPlus"])) y += 1;
  if (y<0) y = 0; if (y>=this.map.height) y = this.map.height-1;
  if (IsKeyPressed(this.keys["zMinus"])) z -= 1;
  if (IsKeyPressed(this.keys["zPlus"])) z += 1;
  if (z<0) z = 0; //if (z>=map.layers.length) z = map.layers.length-1;
  if (IsKeyPressed(this.keys["add"])) {
   if (!this.map.layers[z]) this.map.layers[z] = [];
   if (!this.map.layers[z][x]) this.map.layers[z][x] = [];
   this.map.layers[z][x][y] = this.o.col.r+","+this.o.col.g+","+this.o.col.b+","+this.o.type;
   }
  if (IsKeyPressed(this.keys["del"])) {
   if (this.map.layers[z])
    if (this.map.layers[z][x])
     if (this.map.layers[z][x][y])
      delete this.map.layers[z][x][y];
   }
  if (IsKeyPressed(KEY_RIGHT)) g.camX += 1;
  if (IsKeyPressed(KEY_LEFT)) g.camX -= 1;
  if (IsKeyPressed(KEY_UP)) g.camY -= 1;
  if (IsKeyPressed(KEY_DOWN)) g.camY += 1;
  if (IsKeyPressed(this.keys["zoomIn"])) g.zoom += .01;
  if (IsKeyPressed(this.keys["zoomOut"])) g.zoom -= .01;
  if (IsKeyPressed(KEY_D)) this.debug = !this.debug;
  if (IsKeyPressed(KEY_H)) this.help = !this.help;
  if (IsKeyPressed(KEY_S)) this.save(fn);
  if (IsKeyPressed(KEY_W)) {this.save(fn); this.done = true;}
  if (IsKeyPressed(KEY_Q)) this.done = true;
  if (IsAnyKeyPressed()) ClearKeyQueue();
  }
 this.done = false;
 }