RequireScript("a_req.js");
RequireSystemScript("colors.js");
/*******************************************************************************
Font(f,col) - container object for a base font
 f - font to use
 col - color to apply

METHODS
setFont(f) - set the font
setColorMask(col) - set the font's color
getColorMask()
getHeight()
getStringWidth()
getStringHeight()
drawText(x,y,txt,halign, valign) - draw text aligned
drawText_new(x,y,txt,halign,valign,start) -
 draw text aligned letter-by-letter starting from start
drawZoomedText()
drawTextBox()
draw()
clone()
addBreaks(txt, w) - break text w/hard line breaks
input(x,y, len) - input text at [x,y] w/max length len

PROPERTIES
font - the font
color - the font's color
*******************************************************************************/
function Font(f,col) {
 this.font = (IsFont(f))?f:((IsString(f)?LoadFont(f):GetSystemFont()));
 if (col&&col.alpha!=0) this.setColorMask(col);
 else this.setColorMask();
 this.images = [];
 for (var i=0; i<256; i++)
  this.images.push(this.font.getCharacterImage(i));
 }
Font.prototype.setFont = function(f) {
 this.font = (IsFont(f))?f:((IsString(f)?LoadFont(f):GetSystemFont()));
 }
Font.prototype.setColorMask = function(col) {
 this.color = (IsColor(col))?col:CreateColor(255,255,255);
 }
Font.prototype.getColorMask = function() {
 if (this.color) return this.color;
 else return CreateColor(255,255,255);
 }
Font.prototype.getHeight = function() {
 return this.font.getHeight();
 }
Font.prototype.getStringWidth = function(txt,noWS) {
 if (!noWS)
  return this.font.getStringWidth(txt);
 var ret = 0;
 for (var i=0; i<txt.length; i++) {
  if (txt[i]!="\n") ret += this.font.getStringWidth(txt[i]);
  }
 return ret;
 }
Font.prototype.getStringHeight = function(txt,w) {
 if (!w||w<this.getSmallCharWidth()) w = this.font.getStringWidth(txt);
 return this.font.getStringHeight(txt,w);
 }
Font.prototype.getCharacterImage = function(i) {
 if (i>255||i<0) Abort("Font.getCharacterImage failed ("+i+" not in range)");
 return this.images[i];
 }
Font.prototype.getSmallCharWidth = function() {
 var w = GetScreenWidth();
 for (var i=32; i<128; i++) {
  var ch = String.fromCharCode(i);
  var fw = this.font.getStringWidth(ch);
  if (fw<w) w = fw;
  }
 return w;
 }
Font.prototype.getLargeCharWidth = function() {
 var w = 0;
 for (var i=32; i<128; i++) {
  var ch = String.fromCharCode(i);
  var fw = this.font.getStringWidth(ch);
  if (fw>w) w = fw;
  }
 return w;
 }
//getCharX(str, n, w)
// str - string to search
// n - 0-indexed char to chk
// w - width to split string
Font.prototype.getCharOffsetX = function(str, n, w) {
 if (!w||w<this.getSmallCharWidth()) return this.getStringWidth(str.substring(0,n));
 const h = this.getHeight();
 var dx = 0, dy = 0;	// delta x and y from starting position
 var word = "", ww = 0, lw = 0;
 var ret = 0, chkRet = true;
 for (var i=0; i<str.length; i++) {
  if (i==n) {
   ret = dx+ww;
   lw = ww;
   chkRet = false;
   }
  var ch = str.charAt(i);
  var cw = this.getStringWidth(ch);
  switch (ch) {
   case '\n':
    dx = 0; dy += h;
    word = ""; ww = 0;
    break;
   case ' ':
    if (dx + ww + cw > w) {	// if the word starting from dx goes past w
     dx = ww+cw;
     dy += h;
     }
    else dx += ww+cw;
    word = ""; ww = 0;
    break;
   case '\t':
    if (dx + ww + cw > w) {
     dx = ww+cw;
     dy += h;
     }
    else dx += ww+cw;
    word = ""; ww = 0;
    break;
   default:  // if we've gone over the limit and dx = 0, draw the old word and split the new one off
    if (ww + cw > w && dx == 0) {
     dy += h; word = ""; ww = 0;
     }
    else if (dx + ww + cw > w) {
     dx = 0; dy += h;
     }
    word += ""+ch; ww = this.getStringWidth(word);
    break;
   }
  if (word==""&&!chkRet) {
   if (ch!="\n")
    ret = dx-this.getStringWidth(str.substring(n,i+1));
   //else ret = lw;
   chkRet = true;
   }
  }
 return ret;
 }

//getCharY(str, n, w)
// str - string to search
// n - 0-indexed char to chk
// w - width to split string
Font.prototype.getCharOffsetY = function(str, n, w) {
 if (!w||w<this.getSmallCharWidth()) return 0;
 const h = this.getHeight();
 var dx = 0, dy = 0;	// delta x and y from starting position
 var word = "", ww = 0;
 var ret = 0, chkRet = true;
 for (var i=0; i<str.length; i++) {
  if (i==n) {
   ret = dy;
   chkRet = false;
   }
  var ch = str.charAt(i);
  var cw = this.getStringWidth(ch);
  switch (ch) {
   case '\n':
    dx = 0; dy += h;
    word = ""; ww = 0;
    break;
   case ' ':
    if (dx + ww + cw > w) {	// if the word starting from dx goes past w
     dx = ww+cw;
     dy += h;
     }
    else dx += ww+cw;
    word = ""; ww = 0;
    break;
   case '\t':
    if (dx + ww + cw > w) {
     dx = ww+cw;
     dy += h;
     }
    else dx += ww+cw;
    word = ""; ww = 0;
    break;
   default:  // if we've gone over the limit and dx = 0, draw the old word and split the new one off
    if (ww + cw > w && dx == 0) {
     dy += h; word = ""; ww = 0;
     }
    else if (dx + ww + cw > w) {
     dx = 0; dy += h;
     }
    word += ""+ch; ww = this.getStringWidth(word);
    break;
   }
  if (word==""&&!chkRet) {
   if (ch!="\n") ret = dy;
   chkRet = true;
   }
  }
 return ret;
 }
Font.prototype.drawText = function(x,y, txt, halign,valign) {
 var xoff = 0, yoff = 0;
 if (!halign) halign = "left";
 if (!valign) valign = "top";
 switch (halign.toLowerCase()) {
  case "right": xoff = this.font.getStringWidth(txt); break;
  case "center": xoff = this.font.getStringWidth(txt)/2; break;
  default: xoff = 0;
  }
 switch (valign.toLowerCase()) {
  case "bottom": yoff = this.font.getHeight(); break;
  case "middle": yoff = this.font.getHeight()/2; break;
  default: yoff = 0;
  }
 var oldC = this.font.getColorMask();
 if (this.color) this.font.setColorMask(this.color);
 this.font.drawText(x-xoff,y-yoff,txt);
 this.font.setColorMask(oldC);
 }
Font.prototype.drawText_new = function(x,y, txt, halign,valign) {
 var xoff = 0, yoff = 0;
 switch (halign) {
  case "right": xoff = this.font.getStringWidth(txt); break;
  case "center": xoff = this.font.getStringWidth(txt)/2; break;
  default: xoff = 0;
  }
 switch (valign) {
  case "bottom": yoff = this.font.getHeight(); break;
  case "middle": yoff = this.font.getHeight()/2; break;
  default: yoff = 0;
  }
 var oldC = this.font.getColorMask();
 this.font.setColorMask(this.color);
 var img = ScrToImage();
 for (var i=0; i<txt.length; i++) {
  img.blit(0,0);
  this.font.drawText(x-xoff,y-yoff,txt.substr(0,i));
  FlipScreen();
  }
 //Drawtext_new(this.font, x-xoff,y-yoff,txt);
 img.blit(0,0);
 this.font.drawText(x-xoff,y-yoff,txt);
 this.font.setColorMask(oldC);
 }
Font.prototype.drawZoomedText = function(x,y,z, txt) {
 this.font.drawZoomedText(x,y,z,txt);
 }
Font.prototype.drawTextBox = function(x,y,w,h,off,txt,align) {
 if (!align) align = "left";
 switch (align.toLowerCase()) {
  case "center": case "right": align = align.toLowerCase(); break;
  default: align = "left";
  }
 var t = this.addBreaks(txt,w);
 if (align=="left") {
  this.font.drawTextBox(x,y,w,h,off,t);
  return;
  }
 var a = t.split("\n");
 var xoff = 0;
 if (align=="center") xoff = w/2;
 else if (align=="right") xoff = w;
 var r = GetClippingRectangle();
 SetClippingRectangle(x,y,w,h);
 for (var i=0; i<a.length; i++)
  this.drawText(x+xoff,y+off+i*this.getHeight(), a[i], align);
 SetClippingRectangle(r.x,r.y,r.width,r.height);
 }
Font.prototype.drawTextBox_new = function(x,y,w,h,off,txt,align) {
 if (!align) align = "left";
 switch (align.toLowerCase()) {
  case "center": case "right": align = align.toLowerCase(); break;
  default: align = "left";
  }
 var t = this.addBreaks(txt,w);
 var a = t.split("\n");
 var xoff = 0;
 if (align=="center") xoff = w/2;
 else if (align=="right") xoff = w;
 var r = GetClippingRectangle();
 for (var i=0; i<a.length; i++)
  this.drawText_new(x,y+off+i*this.getHeight(),a[i],align);
 SetClippingRectangle(r.x,r.y,r.width,r.height);
 }
Font.prototype.drawItalicText = function(x,y, txt) {
 for (var i=0; i<txt.length; i++) {
  var img = this.getCharacterImage(txt.charCodeAt(i));
  var offx = this.getCharOffsetX(txt,i);
  if (this.color==CreateColor(255,255,255))
   blitSkew(img,x+offx,y,15);
  else
   blitSkewMask(img,x+offx,y,15,this.color);
  }
 }
Font.prototype.drawItalicTextBox = function(x,y, w,h, txt) {
 var sArr = this.addBreaks(txt,w);
 sArr = sArr.split("\n");
 for (var i=0; i<sArr.length; i++) {
  this.drawItalicText(x,y+i*this.getHeight(), sArr[i]);
  }
 }
Font.prototype.drawTextParsed = function(x,y, nt_obj) {
 var txt = nt_obj.stripped;
 var bgc = null, bCol = this.getColorMask(), col = bCol;
 var offX = 0, ch = this.getHeight(), cX = x;
 if (nt_obj.tags["font"]) {
  var q = 0;
  while (q<nt_obj.tags["font"].length) {
   var props = nt_obj.tags["font"][q].props;
   if (props["bgcolor"]) {	// do bgcoloring
    var p = props["bgcolor"];
    if (IsHex(p,6)) {
     var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
     bgc = CreateColor(r,g,b);
     }
    else if (IsHex(p,8)) {
     var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
     var a = hex2dec(p.substr(6,2));
     bgc = CreateColor(r,g,b,a);
     }
    else eval("bgc = "+p);
    if (bgc) {
     var cw = this.getStringWidth(txt.substring(nt_obj.tags["font"][q].start,nt_obj.tags["font"][q].end));
     cX = x+this.getCharOffsetX(txt,q);
     Rectangle(cX,y, cw,ch,bgc);
     }
    }
   q++;
   }
  }
 cX = x;
 for (var i=0; i<txt.length; i++) {
  var c = txt.charAt(i);
  if (c=="\n") continue;
  bgc = null; col = bCol;
  var it = false, bold = false;
  var u = false, o = false, t = false;
  var cw = this.getStringWidth(c);
  cX += offX;
  if (nt_obj.tags["font"]) {
   for (var j=0; j<nt_obj.tags["font"].length; j++) {
    if (nt_obj.tags["font"][j].isOpen(i)) {
     var props = nt_obj.tags["font"][j].props;
     if (props["bold"]) {		// do boldfacing
      var p = props["bold"];
      if (p=="true"||p==1||p!="false") bold = true;
      }
     if (props["italic"]) {		// do italicizing
      var p = props["italic"];
      if (p=="true"||p==1||p!="false") it = true;
      }
     if (props["color"]) {		// do coloring
      var p = props["color"];
      if (IsHex(p,6)) {
       var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
       newCol = CreateColor(r,g,b);
       }
      else if (IsHex(p,8)) {
       var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
       var a = hex2dec(p.substr(6,2));
       newCol = CreateColor(r,g,b,a);
       }
      else eval("newCol = "+p);
      if (col!=newCol) col = newCol;
      }
     if (props["decor"]) {		// do decorating
      var p = props["decor"].split("|");
      for (var z=0; z<p.length; z++) {
       if (p[z]=="uline") u = true;
       else if (p[z]=="oline") o = true;
       else if (p[z]=="thru") t = true;
       }
      }
     }
    }
   }
  var col2 = CreateColor(col.red,col.green,col.blue,col.alpha-64);
  var col3 = CreateColor(col.red-64,col.green-64,col.blue-64);
  if (bold) {
   this.setColorMask(col2);
   if (it) this.drawItalicText(cX+2,y+2,c);
   else this.drawText(cX+2,y+2,c);
   }
  this.setColorMask(col);
  if (it) this.drawItalicText(cX,y,c);
  else this.drawText(cX,y,c);
  if (u) Line(cX,y+ch-1, cX+cw-1,y+ch-1, col3);
  if (o) Line(cX,y-1, cX+cw-1,y-1, col3);
  if (t) Line(cX,y+ch/2, cX+cw-1,y+ch/2, col3);
  offX = cw;
  }
 }
Font.prototype.drawTextBoxFiltered = function(x,y, w,h, off, nt_obj) {
 this.drawTextBox(x,y, w,h, off, nt_obj.stripped);
 }
//drawTextBoxParsed():
//supported tags -
// <font></font>
//future tags -
// <hr/>
//supported <font> properties -
// bgcolor(color ^ hex6 ^ hex8)
// italic(bool)
// bold(bool)
// color(color ^ hex6 ^ hex8)
// decor(uline | oline | thru)
//future <font> properties -
// size
Font.prototype.drawTextBoxParsed = function(x,y, w,h, nt_obj) {
 var txt = nt_obj.stripped;
 var bgc = null, col = this.getColorMask();
 var offX = 0, offY = 0, ch = this.getHeight();
 var spl = this.addBreaks(txt,w,true);
 var rect = GetClippingRectangle();
 SetClippingRectangle(x,y,w,h);
 if (nt_obj.tags["font"]) {
  var q = 0;
  while (q<nt_obj.tags["font"].length) {
   var props = nt_obj.tags["font"][q].props;
   if (props["bgcolor"]) {	// do bgcoloring
    var p = props["bgcolor"];
    if (IsHex(p,6)) {
     var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
     bgc = CreateColor(r,g,b);
     }
    else if (IsHex(p,8)) {
     var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
     var a = hex2dec(p.substr(6,2));
     bgc = CreateColor(r,g,b,a);
     }
    else eval("bgc = "+p);
    if (bgc) {
     var g = nt_obj.tags["font"][q].start;
     while (g<=nt_obj.tags["font"][q].end) {
      var cX = x+this.getCharOffsetX(txt,g,w);
      var cY = y+this.getCharOffsetY(txt,g,w);
      var cw = this.getStringWidth(txt[g]);
      if (txt[g]!="\n"&&!(txt[g]==" "&&cY!=y+this.getCharOffsetY(txt,g+1,w)))
       Rectangle(cX,cY, cw,ch,bgc);
      g++;
      }
     }
    }
   q++;
   }
  }
 var img = ScrToImage();
 for (var i=0; i<txt.length; i++) {
  img.blit(0,0);
  var c = txt.charAt(i);
  if (c=="\n") continue;
  bgc = null; col = White;
  var cw = this.getStringWidth(c);
  offX = this.getCharOffsetX(txt,i,w);
  offY = this.getCharOffsetY(txt,i,w);
  var it = false, bold = false;
  var u = false, o = false, t = false;
  var cX = x, cY = y;
  if (offX) cX += offX; if (offY) cY += offY;
  if (nt_obj.tags["font"]) {
   var j = 0;
   while (j<nt_obj.tags["font"].length) {
    if (nt_obj.tags["font"][j].isOpen(i)) {
     var props = nt_obj.tags["font"][j].props;
     if (props["italic"]) {		// do italicizing
      var p = props["italic"];
      if (p=="true"||p==1||p!="false") it = true;
      }
     if (props["bold"]) {		// do boldfacing
      var p = props["bold"];
      if (p=="true"||p==1||p!="false") bold = true;
      }
     if (props["color"]) {		// do coloring
      var p = props["color"];
      if (IsHex(p,6)) {
       var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
       newCol = CreateColor(r,g,b);
       }
      else if (IsHex(p,8)) {
       var r = hex2dec(p.substr(0,2)), g = hex2dec(p.substr(2,2)), b = hex2dec(p.substr(4,2));
       var a = hex2dec(p.substr(6,2));
       newCol = CreateColor(r,g,b,a);
       }
      else eval("newCol = "+p);
      if (col!=newCol) col = newCol;
      }
     if (props["decor"]) {		// do decorating
      var p = props["decor"].split("|");
      for (var z=0; z<p.length; z++) {
       if (p[z]=="uline") u = true;
       else if (p[z]=="oline") o = true;
       else if (p[z]=="thru") t = true;
       }
      }
     }
    j++;
    }
   }
  var col2 = CreateColor(col.red,col.green,col.blue,col.alpha-96);
  var col3 = CreateColor(col.red-64,col.green-64,col.blue-64);
  if (bold) {
   this.setColorMask(col2);
   if (it) this.drawItalicText(cX+2,cY+2,c);
   else this.drawText(cX+2,cY+2,c);
   }
  this.setColorMask(col);
  if (it) this.drawItalicText(cX,cY,c);
  else this.drawText(cX,cY,c);
  if (u) Line(cX,cY+ch-1, cX+cw,cY+ch-1, col3);
  if (o) Line(cX,cY, cX+cw-1,cY, col3);
  if (t) Line(cX,cY+ch/2, cX+cw-1,cY+ch/2, col3);
  img = ScrToImage();
  FlipScreen();
  }
 img.blit(0,0);
 SetClippingRectangle(rect.x,rect.y,rect.width,rect.height);
 }
Font.prototype.draw = function(x,y,txt) {
 var w = this.getStringWidth(txt);
 if (x+w>GetScreenWidth()) w = GetScreenWidth()-x;
 this.font.drawTextBox(x,y,w,this.getStringHeight(txt,w),0,txt);
 }
Font.prototype.clone = function() {
 var f = this.font;
 var c = (this.color)?this.color:undefined;
 var ret = new Font(f,c);
 return ret;
 }
// addBreaks -
//  attempts to break txt w/newlines according to w
//  we don't like tiny or too large w values, so
//  we limit w to between the width of the smallest
//  displayable char (not including \t or \n) and
//  the width of the screen; if we put an x param
//  w would take into account x as well
// the how:
//  for() loop thru txt, temp word adds next char
//  if char isn't space, tab, or newline; if
//  total line width > w, add temp word to ret
//  then add newline, then clear temp word;
//  else add next char to temp word regardless
Font.prototype.addBreaks = function(txt, w, keepWS) {
 /*
const h = this.getHeight();
 // delta x and y from starting position
 var dx = 0, dy = 0;
 var word = "";
 var ww = 0;
 for (var i=0; i<n; i++) {
  var ch = str.charAt(i);
  var cw = this.getStringWidth(ch);
  if (ch == ' ') {          // if it's a space, draw the word
   if (dx + ww + cw > w) {
    dx = ww + cw;
    dy += h;
    }
   else dx += ww+cw;
   word = "";
   ww = 0;
   }
  else if (ch == '\t') {  // if it's a tab, draw the word
   if (dx + ww + cw > w) {
    dx = ww+cw
    dy += h;
    }
   else dx += ww+cw;
   word = "";
   ww = 0;
   }
  else if (ch == '\n') {  // newline time, awww yeah
   dx = 0;
   dy += h;
   word = "";
   ww = 0;
   }
  else {
   // if we've gone over the limit and dx = 0, draw the old word and split the new one off
   if (ww + cw > w && dx == 0) {
    dy += h;
    word = "";
    ww = 0;
    }
   else if (dx + ww + cw > w) {
    dx = 0;
    dy += h;
    }
   word += ch;
   ww += cw;
   }
  }
 return dy;

 const int space_width = GetStringWidth(" ");
  const int tab_width   = GetStringWidth("   ");
  const int max_height  = GetMaxHeight();
  const char* p = text;
  int old_x, old_y, old_w, old_h;
  GetClippingRectangle(&old_x, &old_y, &old_w, &old_h);
  SetClippingRectangle(std::max(x, old_x), std::max(y, old_y), std::min(w, old_w), std::min(h, old_h));
  y += offset;
  // delta x and y from starting position
  int dx = 0;
  int dy = 0;
  std::string word;
  int word_width = 0;
  int range = m_Font.GetNumCharacters();
  // parse the text into words
  while (*p) {

    unsigned char ch = (unsigned char) *p;
    if ((int)ch < 0 || (int)ch >= range) { p++; continue; }

    if (ch == ' ') {          // if it's a space, draw the word
      
      if (dx + word_width + space_width > width) {
        dx = word_width + space_width;
        dy += max_height;
      } else {
        dx += word_width + space_width;
      }

      word.resize(0);
      word_width = 0;

    } else if (ch == '\t') {  // if it's a tab, draw the word

      if (dx + word_width + tab_width > width) {
        dx = word_width + tab_width;
        dy += max_height;
      } else {
        dx += word_width + tab_width;
      }

      word_width = 0;

    } else if (ch == '\n') {  // newline time, awww yeah

      dx = 0;
      dy += max_height;
      word.resize(0);
      word_width = 0;

    } else {

      int char_width = m_Font.GetCharacter(ch).GetWidth();

      // if we've gone over the limit and dx = 0, draw the old word and split the new one off
      if (word_width + char_width > width && dx == 0) {
        dy += max_height;
        word.resize(0);
        word_width = 0;
      } else if (dx + word_width + char_width > width) {
        dx = 0;
        dy += max_height;
      }

      word += ch;
      word_width += char_width;

    }

    p++;
  }

  // add a line more to give the total line height since this
  // algorithm was supposed to draw text!
  dy += max_height;
  return dy;
*/
 
 
 
 
 if (!keepWS) keepWS = false;
 if (w<this.getSmallCharWidth()) w = this.getSmallCharWidth();
 if (w>GetScreenWidth()) w = GetScreenWidth();
 var ret = "", dx = 0, w_wd = 0, word = "";
 for (var i=0; i<txt.length; i++) {	// fix loop to handle line-by-line if needed
  var ch = txt.charAt(i);
  var w_ch = this.getStringWidth(ch);
  switch (ch) {
   case "\n": ret += word;
    dx = 0; ret += ch;
    word = ""; w_wd = 0;
    break;
   case " ":
    ret += word;
    if (dx+w_wd+w_ch>w) {dx = w_wd+w_ch; ret += ch; ret += "\n";}
    else {dx += w_wd+w_ch; ret += ch;}
    word = ""; w_wd = 0;
    break;
   case "\t":
    ret += word; if (keepWS) ret += ch;
    if (dx+w_wd+w_ch>w) {dx = w_ch; ret += "\n";}
    else {dx += w_wd+w_ch; ret += ch;}
    word = ""; w_wd = 0;
    break;
   default:
    if (w_wd+w_ch>w&&dx==0) {ret += word; ret += "\n"; word = ""; w_wd = 0;}
    else if (dx+w_wd+w_ch>w) {dx = 0; ret += "\n";}
    word += ch; w_wd += w_ch;
    break;
   }
  }
 if (word!="") ret += word;
 return ret;
 }
Font.prototype.breakIt = function(txt,w) {
 var ret = "", w = 0;
 for (var i=0; i<txt.length; i++) {
  var c = txt.charAt(i);
  var cw = this.getStringWidth(cw);
  if (w+cw>w) {ret += "\n"; w = cw;}
  else if (c=="\n") {ret += "\n"; w = 0;}
  else {
   ret += ""+c;
   w += cw;
   }
  }
 return ret;
 }
Font.prototype.input = function(x,y, len) {
 var bg = ScrToImage();
 var cursor = CreateSurface(2,this.getHeight(),this.getColorMask()).createImage();
 var str = "";
 var pos = 0;
 // No max given, default to 256
 if (!IsInt(len)) max_chars = 256;
 while (true) {
  bg.blit(0, 0);
  this.drawText(x,y, str);
  if (Math.sin(GetTime()>>8)>0)
   cursor.blit(x+this.getStringWidth(str.slice(0,pos)), y);
  FlipScreen();
  while (AreKeysLeft()) {
   var key = GetKey();
   switch (key) {
    case KEY_ENTER: return str; break;
    case KEY_BACKSPACE:
     if (str!="") str = str.slice(0, pos-1) + str.slice(pos);
     if (pos>0) pos--;
     break;
    case KEY_DELETE: if (str!="") str = str.slice(0, pos) + str.slice(pos+1); break;
    case KEY_LEFT: if (pos>0) pos--; break;
    case KEY_RIGHT:
     if (pos<=str.length-1) pos++; break;
    case KEY_HOME: pos = 0; break;
    case KEY_END: pos = str.length; break;
    default:
     var shift = IsKeyPressed(KEY_SHIFT);
     if (GetKeyString(key, shift) != "" && (str.length < len)) {
      str = str.slice(0, pos) + GetKeyString(key, shift) + str.slice(pos);
      pos++;
      }
    } // end switch
   } // end while (keys left)
  } // end while (true)
 }
