function ntml(strn, def) {
 this.def = def;		// current ntml def
 this.orig = strn;		// original string
 this.tags = [];		// tag array
 this.usedtags = [];	// tags that are used
 
 //parse time
 var reg = RegExp("<(.+?)>","g");	// the regexp to search for delimiters
 var exp = RegExp("^[^/]+$");		// the regexp to search for non-close tags
 var ex1 = RegExp("^[^/]+");		// the regexp to search for possible self-closed tags
 var ex2 = RegExp("^/[^/]+");		// the regexp to search for <non-super> close tags
 var ex3 = RegExp("^//.+");			// the regexp to find super-close tags
 var offsets = [];
 var replace_num = 0;
 this.stripped = strn.replace(reg,
  function(str, p1, offset, s) {
  offsets.push(offset-replace_num);
  replace_num += str.length;
  return "";
  });		// strip the text of markup and add its index to offsets
 
 this.tagindex = [];	// will hold delimited text
 for (var i=0; i<offsets.length; i++) {	// go thru offsets
  var tagArr = reg.exec(strn);		// find occurances of the regexp
  var tag = tagArr[1].split(" ");	// object properties are separated by spaces
  if (tag.length>1) {		// if object has props
   var props = "";
   for (var j=1; j<tag.length; j++) {
    props += ""+tag[j];
    if (j!=tag.length-1) props += " ";
    }
   if (exp.test(tag[0])) {	// if not a closing tag
    if (props.charAt(props.length-1)=="/")
     this.open(tag[0], props.substr(0, props.length-1), offsets[i], true);	// self-closing
    else
     this.open(tag[0], props, offsets[i], false);
    }
   }
  else {
   if (ex1.test(tag[0])) {
    if (tag[0].charAt(tag[0].length-1)=="/")
     this.open(tag[0], null, offsets[i], true);
    else this.open(tag[0], null, offsets[i], false);
    }
   else if (ex2.test(tag[0]))
    this.close(tag[0].substr(1), offsets[i]);	// close it if non-super closed
   else if (ex3.test(tag[0]))
    this.closeMaster(tag[0].substr(2), offsets[i]);	// super-close it
   else Abort(tag[0]);
   }
  }
 
 // close any remaining open tags
 for (var i=0; i<this.usedtags.length; i++)
  this.closeMaster(this.usedtags[i], this.stripped.length);
 }
ntml.prototype.open = function(tag, prop, iSt, isSelfContained) {
 if (this.def) {
  var matched = this.def.match(tag);
  if (matched) tag = matched;	// if a definition exists, switch this tag to its alias
  }
 if (!this.tags[tag]) this.tags[tag] = [];	// if this tag isn't in ntml, add it
 this.tags[tag].push(new ntml_tag(this, tag, prop, iSt, isSelfContained));
 this.tagindex.push(this.tags[tag][this.tags[tag].length-1]);
 for (var i=0; i<this.usedtags.length; i++) {
  if (this.usedtags[i]==tag) return true;
  }
 this.usedtags.push(tag);	// add this tag to the queue of tags that need to be closed
 return true;
 }
ntml.prototype.close = function(tag, iEnd) {
 if (this.def) {
  var matched = this.def.match(tag);
  if (matched) tag = matched;
  }
 for (var i=this.tags[tag].length-1; i>=0; i--) {
  if (!this.tags[tag][i].end) {		// close the most recent open tag
   this.tags[tag][i].end = iEnd;
   return true;
   }
  }
 return true;
 }
ntml.prototype.closeMaster = function(tag, iEnd) {
 if (this.def) {
  var matched = this.def.match(tag);
  if (matched) tag = matched;
  }
 for (var i=0; i<this.tags[tag].length; i++) {	// close ALL of tag
  if (!this.tags[tag][i].end) this.tags[tag][i].end = iEnd;
  }
 }
ntml.prototype.dumpLog = function(lf) {
 var ret = "\n\n";
 ret += "str::orig = "+this.orig+"\n";
 ret += "str::stripped = "+this.stripped+"\n\n";
 if (this.def) {	// DEFINITIONS DUMP
  ret += "###DEFINITION###\n";
  for (var i=0; i<this.def.tags.length; i++) {
   ret += "TAG::";
   for (var j=0; j<this.def.tags[i].length; j++)
    ret += this.def.tags[i][j]+" ";
   ret += "\n";
   }
  ret += "\n";
  }
 for (var i=0; i<this.usedtags.length; i++) {	// USED TAGS DUMP
  var used = this.usedtags[i];
  var tag = this.tags[used];
  ret += "TAGINST::"+used+":\n";
  for (var j=0; j<tag.length; j++) {
   ret += j+":\t["+tag[j].start+"-"+tag[j].end+"] (";
   //if (tag[j].prop) ret += tag[j].prop;
   if (tag[j].props) {
    var len = 0;
    for (var k in tag[j].props) {
     ret += k+"='"+tag[j].props[k]+"', ";
     len++;
     }
    ret += "PROPS:"+len;
    }
   else ret += "NO PROPERTIES";
   ret += ") ";
   if (tag[j].start==tag[j].end) ret += "SELF-CONTAINED\n";
   else ret += "TEXT="+tag[j].getText()+"\n";
   }
  if (i!=this.usedtags.length-1) ret += "\n";
  }
 ret += "\nTAG INDICES:\n";		// TAG INDICES DUMP
 for (var i=0; i<this.tagindex.length; i++) {
  var tag = this.tagindex[i];
  ret += i+":\t["+tag.start+"-"+tag.end+"] (";
  //if (tag.prop) ret += tag.prop;
  if (tag.props) {
   var len = 0;
   for (var k in tag.props) {
    ret += k+"='"+tag.props[k]+"', ";
    len++
    }
   ret += "PROPS:"+len;
   }
  else ret += "NO PROPERTIES";
  ret += ") ";
  if (tag.start==tag.end) ret += "SELF-CONTAINED\n";
  else ret += "TEXT="+tag.getText()+"\n";
  }
 if (!lf) lf = "ntmldump.log";
 var log = OpenLog(lf);
 log.beginBlock(GetTime());
 log.write(ret);
 log.endBlock();
 }

function ntml_tag(parent, tag, prop, iSt, isSelfContained) {
 this.parent = parent;	// PARENT NTML OBJ
 this.tag = tag;
 this.prop = prop;		// TAG PROPERTIES
 this.props = null;
 if (this.prop) {
  var props = this.prop.split(" ");
  this.props = [];		// TAG PROPERTY ASSOCIATIVE ARRAY
  for (var i=0; i<props.length; i++) {
   var curr = props[i].split("=");
   this.props[curr[0]] = curr[1];
   }
  }
 this.start = iSt;		// START INDEX
 this.selfcontained = isSelfContained;
 this.end = (isSelfContained)?iSt:null;
 }
ntml_tag.prototype.isOpen = function(loc) {
 if (loc>=this.start&&loc<=this.end-1) return true;
 return false;
 }
ntml_tag.prototype.getText = function() {
 return this.parent.stripped.substring(this.start,this.end);
 }

function ntml_def() {
 this.tags = [];
 }
ntml_def.prototype.add = function(orig) {
 // add taglist as aliases of orig
 var l = this.tags.length;
 this.tags[l] = [];
 for (var i=0; i<arguments.length; i++)
  this.tags[l][i] = arguments[i];
 }
ntml_def.prototype.match = function(tag) {
 for (var i=0; i<this.tags.length; i++) {
  for (var j=0; j<this.tags[i].length; j++) {
   if (tag==this.tags[i][j]) return this.tags[i][0];
   }
  }
 return false;
 }
ntml_def.prototype.exec = function(str) {
 return new ntml(str,this);
 }